@@ -34,7 +34,10 @@ logger = setup_logger(
     "trackme.rest.splk_outliers_engine_user",
     "trackme_rest_api_splk_outliers_engine_user.log",
 )
-# Redirect logger.module calls
+# Redirect global logging to use the same handler
+import logging
+logging.getLogger().handlers = logger.handlers
+logging.getLogger().setLevel(logger.level)
 
 
 # import rest handler
@@ -47,9 +50,9 @@ from trackme_libs import trackme_getloglevel, run_splunk_search
 import splunklib.client as client
 
 
-class TrackMeHandlerSplkOutliersEngine_v2(trackme_rest_handler.RESTHandler):
+class TrackMeHandlerSplkOutliersEngineRead_v2(trackme_rest_handler.RESTHandler):
     def __init__(self, command_line, command_arg):
-        super(TrackMeHandlerSplkOutliersEngine_v2, self).__init__(
+        super(TrackMeHandlerSplkOutliersEngineRead_v2, self).__init__(
             command_line, command_arg, logger
         )
 
@@ -138,7 +141,7 @@ class TrackMeHandlerSplkOutliersEngine_v2(trackme_rest_handler.RESTHandler):
             app="trackme",
             port=splunkd_port,
             token=request_info.session_key,
-            timeout=300,
+            timeout=600,
         )
 
         # set loglevel
@@ -258,7 +261,7 @@ class TrackMeHandlerSplkOutliersEngine_v2(trackme_rest_handler.RESTHandler):
             app="trackme",
             port=splunkd_port,
             token=request_info.session_key,
-            timeout=300,
+            timeout=600,
         )
 
         # set loglevel
@@ -389,7 +392,7 @@ class TrackMeHandlerSplkOutliersEngine_v2(trackme_rest_handler.RESTHandler):
             app="trackme",
             port=splunkd_port,
             token=request_info.session_key,
-            timeout=300,
+            timeout=600,
         )
 
         # set loglevel
@@ -518,7 +521,7 @@ class TrackMeHandlerSplkOutliersEngine_v2(trackme_rest_handler.RESTHandler):
             app="trackme",
             port=splunkd_port,
             token=request_info.session_key,
-            timeout=300,
+            timeout=600,
         )
 
         # set loglevel
@@ -694,7 +697,7 @@ class TrackMeHandlerSplkOutliersEngine_v2(trackme_rest_handler.RESTHandler):
             app="trackme",
             port=splunkd_port,
             token=request_info.system_authtoken,
-            timeout=300,
+            timeout=600,
         )
 
         # set loglevel
@@ -1010,7 +1013,7 @@ class TrackMeHandlerSplkOutliersEngine_v2(trackme_rest_handler.RESTHandler):
             app="trackme",
             port=splunkd_port,
             token=request_info.system_authtoken,
-            timeout=300,
+            timeout=600,
         )
 
         # set loglevel
@@ -1248,7 +1251,7 @@ class TrackMeHandlerSplkOutliersEngine_v2(trackme_rest_handler.RESTHandler):
                 rest_url,
                 params={"output_mode": "json", "count": 0},
                 headers=header,
-                timeout=300,
+                timeout=600,
                 verify=False,
             )
 
