@@ -2,7 +2,7 @@
 # coding=utf-8
 
 __author__ = "TrackMe Limited"
-__copyright__ = "Copyright 2022-2026, TrackMe Limited, U.K."
+__copyright__ = "Copyright 2023-2025, TrackMe Limited, U.K."
 __credits__ = "TrackMe Limited, U.K."
 __license__ = "TrackMe Limited, all rights reserved"
 __version__ = "0.1.0"
@@ -214,6 +214,7 @@ class TrackMeAckTracker(GeneratingCommand):
                     "splk-flx": int(record.get("tenant_flx_enabled", 0)),
                     "splk-fqm": int(record.get("tenant_fqm_enabled", 0)),
                     "splk-wlk": int(record.get("tenant_wlk_enabled", 0)),
+                    "splk-cim": int(record.get("tenant_cim_enabled", 0)),
                 }
 
             # For each enabled tenant, access the Ack KVstore, loop through the records, verify the Ack expiration and update as needed
@@ -234,6 +235,7 @@ class TrackMeAckTracker(GeneratingCommand):
                     "splk-mhm": "mhm",
                     "splk-flx": "flx",
                     "splk-fqm": "fqm",
+                    "splk-cim": "cim",
                     "splk-wlk": "wlk",
                 }
 
