@@ -18,6 +18,7 @@ import re
 import hashlib
 import ast
 import time
+import concurrent.futures
 
 # Logging imports
 import logging
@@ -70,7 +71,10 @@ from trackme_libs_splk_flx import (
 from trackme_libs import trackme_reqinfo
 
 # Import trackMe utils libs
-from trackme_libs_utils import decode_unicode
+from trackme_libs_utils import decode_unicode, get_uuid
+
+# Import trackMe kvstore batch libs
+from trackme_libs_kvstore_batch import batch_update_worker
 
 # import Splunk libs
 from splunklib.searchcommands import (
@@ -142,7 +146,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                 end = True
 
         logging.info(
-            f'context="perf", get_last_seen_collection records, no_records="{len(collection_records)}", run_time="{round((time.time() - get_collection_start), 3)}", collection="{collection_name}"'
+            f'instance_id="{self.instance_id}", context="perf", get_last_seen_collection records, no_records="{len(collection_records)}", run_time="{round((time.time() - get_collection_start), 3)}", collection="{collection_name}"'
         )
 
         return collection_records_dict
@@ -186,7 +190,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                 end = True
 
         logging.info(
-            f'context="perf", get_thresholds_collection records, no_records="{len(collection_records)}", run_time="{round((time.time() - get_collection_start), 3)}", collection="{collection_name}"'
+            f'instance_id="{self.instance_id}", context="perf", get_thresholds_collection records, no_records="{len(collection_records)}", run_time="{round((time.time() - get_collection_start), 3)}", collection="{collection_name}"'
         )
 
         return collection_records_dict
@@ -219,7 +223,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                 end = True
 
         logging.info(
-            f'context="perf", get_disruption_queue_collection records, no_records="{len(collection_records)}", run_time="{round((time.time() - get_collection_start), 3)}", collection="{collection_name}"'
+            f'instance_id="{self.instance_id}", context="perf", get_disruption_queue_collection records, no_records="{len(collection_records)}", run_time="{round((time.time() - get_collection_start), 3)}", collection="{collection_name}"'
         )
 
         return collection_records_dict, collection_records_keys
@@ -255,7 +259,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                 end = True
 
         logging.info(
-            f'context="perf", get_drilldown_searches_collection records, no_records="{len(collection_records)}", run_time="{round((time.time() - get_collection_start), 3)}", collection="{collection_name}"'
+            f'instance_id="{self.instance_id}", context="perf", get_drilldown_searches_collection records, no_records="{len(collection_records)}", run_time="{round((time.time() - get_collection_start), 3)}", collection="{collection_name}"'
         )
 
         return collection_records_dict, collection_records_keys, collection_records_secondary_keys
@@ -291,48 +295,13 @@ class TrackMeSplkFlxParse(StreamingCommand):
                 end = True
 
         logging.info(
-            f'context="perf", get_default_metrics_collection records, no_records="{len(collection_records)}", run_time="{round((time.time() - get_collection_start), 3)}", collection="{collection_name}"'
+            f'instance_id="{self.instance_id}", context="perf", get_default_metrics_collection records, no_records="{len(collection_records)}", run_time="{round((time.time() - get_collection_start), 3)}", collection="{collection_name}"'
         )
 
         return collection_records_dict, collection_records_keys, collection_records_secondary_keys
 
 
     # batch KVstore update
-    def batch_kvstore_update(self, collection_name, collection_dict):
-        logging.debug(
-            f"calling batch_kvstore_update, collection_dict={json.dumps(collection_dict, indent=2)}"
-        )
-        # connect to the KVstore
-        collection = self.service.kvstore[collection_name]
-
-        # batch update/insert
-        batch_update_collection_start = time.time()
-
-        final_records = []
-        # Handle both dictionary and list inputs
-        if isinstance(collection_dict, dict):
-            # loop through the collection dict and add to the list
-            for key, value in collection_dict.items():
-                value["_key"] = key
-                final_records.append(value)
-        elif isinstance(collection_dict, list):
-            # If it's a list, use the records directly
-            final_records = collection_dict
-
-        # process by chunk
-        chunks = [final_records[i : i + 500] for i in range(0, len(final_records), 500)]
-        for chunk in chunks:
-            try:
-                collection.data.batch_save(*chunk)
-            except Exception as e:
-                logging.error(f'KVstore batch failed with exception="{str(e)}"')
-
-        # perf counter for the batch operation
-        logging.info(
-            f'context="perf", batch KVstore update terminated, no_records="{len(final_records)}", run_time="{round((time.time() - batch_update_collection_start), 3)}", collection="{collection_name}"'
-        )
-
-        return True
 
     def normalize_time(self, record):
 
@@ -368,11 +337,11 @@ class TrackMeSplkFlxParse(StreamingCommand):
                 headers=header,
                 data=json.dumps(data, indent=1),
                 verify=False,
-                timeout=300,
+                timeout=600,
             )
             if response.status_code not in (200, 201, 204):
                 error_msg = f'failed to retrieve the tenant metric index, response.status_code="{response.status_code}", response.text="{response.text}"'
-                logging.error(error_msg)
+                logging.error(f'instance_id="{self.instance_id}", {error_msg}')
                 raise Exception(error_msg)
             else:
                 response_data = json.loads(json.dumps(response.json(), indent=1))
@@ -381,7 +350,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
             error_msg = (
                 f'failed to retrieve the tenant metric index, exception="{str(e)}"'
             )
-            logging.error(error_msg)
+            logging.error(f'instance_id="{self.instance_id}", {error_msg}')
             raise Exception(error_msg)
 
         return tenant_trackme_metric_idx
@@ -396,9 +365,17 @@ class TrackMeSplkFlxParse(StreamingCommand):
         )
         log.setLevel(reqinfo["logging_level"])
 
+        # Max multi thread workers
+        max_multi_thread_workers = int(
+            reqinfo["trackme_conf"]["trackme_general"]["max_multi_thread_workers"]
+        )
+
+        # set instance_id
+        self.instance_id = get_uuid()
+
         # log info
         logging.info(
-            f'tenant_id="{self.tenant_id}", context="{self.context}", TrackMeSplkFlxParse is starting'
+            f'tenant_id="{self.tenant_id}", context="{self.context}", instance_id="{self.instance_id}", TrackMeSplkFlxParse is starting'
         )
 
         # get metric index
@@ -411,12 +388,12 @@ class TrackMeSplkFlxParse(StreamingCommand):
         try:
             thresholds_collection = self.get_thresholds_collection()
             logging.debug(
-                f'tenant_id="{self.tenant_id}", thresholds_collection="{json.dumps(thresholds_collection, indent=2)}"'
+                f'instance_id="{self.instance_id}", tenant_id="{self.tenant_id}", thresholds_collection="{json.dumps(thresholds_collection, indent=2)}"'
             )
         except Exception as e:
             thresholds_collection = {}
             logging.error(
-                f'tenant_id="{self.tenant_id}", failed to retrieve the thresholds collection, exception="{str(e)}"'
+                f'instance_id="{self.instance_id}", tenant_id="{self.tenant_id}", failed to retrieve the thresholds collection, exception="{str(e)}"'
             )
 
         # get disruption queue collection
@@ -425,41 +402,41 @@ class TrackMeSplkFlxParse(StreamingCommand):
                 self.get_disruption_queue_collection()
             )
             logging.debug(
-                f'tenant_id="{self.tenant_id}", disruption_queue_collection="{json.dumps(disruption_queue_collection, indent=2)}"'
+                f'instance_id="{self.instance_id}", tenant_id="{self.tenant_id}", disruption_queue_collection="{json.dumps(disruption_queue_collection, indent=2)}"'
             )
         except Exception as e:
             disruption_queue_collection = {}
             disruption_queue_collection_keys = []
             logging.error(
-                f'tenant_id="{self.tenant_id}", failed to retrieve the disruption queue collection, exception="{str(e)}"'
+                f'instance_id="{self.instance_id}", tenant_id="{self.tenant_id}", failed to retrieve the disruption queue collection, exception="{str(e)}"'
             )
 
         # get drilldown searches collection
         try:
             drilldown_searches_collection, drilldown_searches_collection_keys, drilldown_searches_collection_secondary_keys = self.get_drilldown_searches_collection()
             logging.debug(
-                f'tenant_id="{self.tenant_id}", drilldown_searches_collection="{json.dumps(drilldown_searches_collection, indent=2)}"'
+                f'instance_id="{self.instance_id}", tenant_id="{self.tenant_id}", drilldown_searches_collection="{json.dumps(drilldown_searches_collection, indent=2)}"'
             )
         except Exception as e:
             drilldown_searches_collection = {}
             drilldown_searches_collection_keys = []
             drilldown_searches_collection_secondary_keys = set()
             logging.error(
-                f'tenant_id="{self.tenant_id}", failed to retrieve the drilldown searches collection, exception="{str(e)}"'
+                f'instance_id="{self.instance_id}", tenant_id="{self.tenant_id}", failed to retrieve the drilldown searches collection, exception="{str(e)}"'
             )
 
         # get default metrics collection
         try:
             default_metrics_collection, default_metrics_collection_keys, default_metrics_collection_secondary_keys = self.get_default_metrics_collection()
             logging.debug(
-                f'tenant_id="{self.tenant_id}", default_metrics_collection="{json.dumps(default_metrics_collection, indent=2)}"'
+                f'instance_id="{self.instance_id}", tenant_id="{self.tenant_id}", default_metrics_collection="{json.dumps(default_metrics_collection, indent=2)}"'
             )
         except Exception as e:
             default_metrics_collection = {}
             default_metrics_collection_keys = []
             default_metrics_collection_secondary_keys = set()
             logging.error(
-                f'tenant_id="{self.tenant_id}", failed to retrieve the default metrics collection, exception="{str(e)}"'
+                f'instance_id="{self.instance_id}", tenant_id="{self.tenant_id}", failed to retrieve the default metrics collection, exception="{str(e)}"'
             )
 
         # Build header and target URL
@@ -489,6 +466,12 @@ class TrackMeSplkFlxParse(StreamingCommand):
         # default metrics records to be added once we have iterated through the records
         default_metrics_records_to_add = []
 
+        # set task
+        #
+        task_start = time.time()
+        task_instance_id = get_uuid()
+        task_name = "iterate_through_records"        
+
         # Loop in the results
         for record in records:
             # increment
@@ -514,7 +497,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                 flx_object = None
             if not flx_object:
                 logging.error(
-                    f'The field object is mandatory and should be rendered as a part of the search results, the object field could not be found in result="{json.dumps(record, indent=2)}"'
+                    f'instance_id="{self.instance_id}", The field object is mandatory and should be rendered as a part of the search results, the object field could not be found in result="{json.dumps(record, indent=2)}"'
                 )
                 raise Exception(
                     "The field object is mandatory and should be rendered as a part of the search results, the object field could not be found in search results"
@@ -599,7 +582,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                 flx_status = None
             if not flx_status:
                 logging.error(
-                    f'The field status is mandatory and should be rendered as a part of the search results, the status field could not be found in result="{json.dumps(record, indent=2)}"'
+                    f'instance_id="{self.instance_id}", The field status is mandatory and should be rendered as a part of the search results, the status field could not be found in result="{json.dumps(record, indent=2)}"'
                 )
                 raise Exception(
                     "The field status is mandatory and should be rendered as a part of the search results, the status field could not be found in search results"
@@ -716,7 +699,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
 
                 if flx_metrics and not flx_metrics_parsed:
                     logging.error(
-                        f'failed to load the submitted metrics as a JSON object, verify it is properly JSON encoded using single or double quotes delimitors, exception="{flx_metrics_parsed_exception}"'
+                        f'instance_id="{self.instance_id}", failed to load the submitted metrics as a JSON object, verify it is properly JSON encoded using single or double quotes delimitors, exception="{flx_metrics_parsed_exception}"'
                     )
                     flx_metrics = {}
                     flx_metrics_parsed_msg = f'Metrics JSON were submitted but could not be parsed properly, verify the JSON syntax, properties should be enquoted with single or double quotes, exception="{flx_metrics_parsed_exception}"'
@@ -762,7 +745,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                                     parsed_list.append(parsed_item)
                                 except json.JSONDecodeError as je:
                                     logging.warning(
-                                        f'Failed to parse metrics list item: "{item}", error: {str(je)}'
+                                        f'instance_id="{self.instance_id}", Failed to parse metrics list item: "{item}", error: {str(je)}'
                                     )
                                     continue
                             else:
@@ -783,7 +766,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                                     parsed_list.append(parsed_item)
                                 except json.JSONDecodeError as je:
                                     logging.warning(
-                                        f'Failed to parse metrics list item: "{item}", error: {str(je)}'
+                                        f'instance_id="{self.instance_id}", Failed to parse metrics list item: "{item}", error: {str(je)}'
                                     )
                                     continue
                             else:
@@ -798,7 +781,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                         flx_record["metrics_list"] = flx_metrics_list
 
                 except Exception as e:
-                    logging.error(f"Error processing metrics_list: {str(e)}")
+                    logging.error(f'instance_id="{self.instance_id}", Error processing metrics_list: {str(e)}')
                     flx_metrics_list = None
 
             if flx_metrics_list:
@@ -810,7 +793,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
 
                 if not isinstance(flx_metrics_list, list):
                     error_msg = f'The field metrics_list should be a list, value="{flx_metrics_list}"'
-                    logging.error(error_msg)
+                    logging.error(f'instance_id="{self.instance_id}", {error_msg}')
                     flx_metrics_list_parsed_msg = error_msg
                     flx_metrics_list_parsed_exception = error_msg
                 else:
@@ -821,7 +804,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
 
                 if flx_metrics_list and not flx_metrics_list_parsed:
                     error_msg = f'failed to load the submitted metrics_list as a JSON object, verify it is properly JSON encoded using single or double quotes delimitors, exception="{flx_metrics_list_parsed_exception}"'
-                    logging.error(error_msg)
+                    logging.error(f'instance_id="{self.instance_id}", {error_msg}')
                     flx_metrics_list_parsed_msg = error_msg
 
                 else:  # we will verify the last seen metrics record if requested
@@ -856,7 +839,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
 
                             # debug logging
                             logging.debug(
-                                f'tenant_id="{self.tenant_id}", object="{flx_object}", object_id="{flx_sha256}", flx_metrics_list_item_epoch="{flx_metrics_list_item_epoch}" ({time.strftime("%c", time.gmtime(flx_metrics_list_item_epoch))}), record="{json.dumps(flx_metrics_list_item, indent=2)}"'
+                                f'instance_id="{self.instance_id}", tenant_id="{self.tenant_id}", object="{flx_object}", object_id="{flx_sha256}", flx_metrics_list_item_epoch="{flx_metrics_list_item_epoch}" ({time.strftime("%c", time.gmtime(flx_metrics_list_item_epoch))}), record="{json.dumps(flx_metrics_list_item, indent=2)}"'
                             )
 
                             # track the maximum epoch time seen in metrics_list
@@ -875,7 +858,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                                 > collection_last_seen_epoch
                             ):
                                 logging.info(
-                                    f'tenant_id="{self.tenant_id}", object="{flx_object}", object_id="{flx_sha256}", action="skipped", collection_last_seen_epoch="{collection_last_seen_epoch}" ({time.strftime("%c", time.gmtime(collection_last_seen_epoch))}) is bigger than flx_metrics_list_item_epoch="{flx_metrics_list_item_epoch}" ({time.strftime("%c", time.gmtime(flx_metrics_list_item_epoch))}), skipping record="{json.dumps(flx_metrics_list_item, indent=2)}"'
+                                    f'instance_id="{self.instance_id}", tenant_id="{self.tenant_id}", object="{flx_object}", object_id="{flx_sha256}", action="skipped", collection_last_seen_epoch="{collection_last_seen_epoch}" ({time.strftime("%c", time.gmtime(collection_last_seen_epoch))}) is bigger than flx_metrics_list_item_epoch="{flx_metrics_list_item_epoch}" ({time.strftime("%c", time.gmtime(flx_metrics_list_item_epoch))}), skipping record="{json.dumps(flx_metrics_list_item, indent=2)}"'
                                 )
                                 flx_metrics_list.remove(flx_metrics_list_item)
 
@@ -925,7 +908,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
 
                 if "default_threshold" in record and not flx_default_threshold_parsed:
                     logging.error(
-                        f'failed to load the submitted default threshold as a JSON object, verify it is properly JSON encoded using single or double quotes delimitors, exception="{flx_default_threshold_parsed_exception}"'
+                        f'instance_id="{self.instance_id}", failed to load the submitted default threshold as a JSON object, verify it is properly JSON encoded using single or double quotes delimitors, exception="{flx_default_threshold_parsed_exception}"'
                     )
                     flx_default_threshold_parsed_msg = f'Default threshold JSON was submitted but could not be parsed properly, verify the JSON syntax, properties should be enquoted with single or double quotes, exception="{flx_default_threshold_parsed_exception}"'
 
@@ -945,7 +928,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                             or not "condition_true" in threshold
                         ):
                             logging.error(
-                                "the default threshold is missing one or more required properties, metric_name, operator, value, condition_true"
+                                f'instance_id="{self.instance_id}", the default threshold is missing one or more required properties, metric_name, operator, value, condition_true'
                             )
                             flx_default_threshold_parsed_msg = "the default threshold is missing one or more required properties, metric_name, operator, value, condition_true"
                             flx_default_threshold_parsed = False
@@ -978,12 +961,12 @@ class TrackMeSplkFlxParse(StreamingCommand):
                     flx_disruption_min_time_sec = int(flx_disruption_min_time_sec)
                     if flx_disruption_min_time_sec < 0:
                         logging.error(
-                            f'The field disruption_min_time_sec must be a positive integer or zero, value="{flx_disruption_min_time_sec}"'
+                            f'instance_id="{self.instance_id}", The field disruption_min_time_sec must be a positive integer or zero, value="{flx_disruption_min_time_sec}"'
                         )
                         flx_disruption_min_time_sec = None
                 except Exception as e:
                     logging.error(
-                        f'The field disruption_min_time_sec must be a positive integer or zero, value="{flx_disruption_min_time_sec}"'
+                        f'instance_id="{self.instance_id}", The field disruption_min_time_sec must be a positive integer or zero, value="{flx_disruption_min_time_sec}"'
                     )
                     flx_disruption_min_time_sec = None
 
@@ -1101,7 +1084,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                             "alert_upper_breached"
                         )
                         logging.debug(
-                            f'Extracting outliers_metrics, kpi_name="{kpi_name}", alert_lower_breached="{value_alert_lower_breached}", alert_upper_breached="{value_alert_upper_breached}"'
+                            f'instance_id="{self.instance_id}", Extracting outliers_metrics, kpi_name="{kpi_name}", alert_lower_breached="{value_alert_lower_breached}", alert_upper_breached="{value_alert_upper_breached}"'
                         )
 
                     flx_outliers_metrics_parsed_msg = (
@@ -1130,7 +1113,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                                 "alert_upper_breached"
                             )
                             logging.debug(
-                                f'Extracting outliers_metrics, kpi_name="{kpi_name}", alert_lower_breached="{value_alert_lower_breached}", alert_upper_breached="{value_alert_upper_breached}"'
+                                f'instance_id="{self.instance_id}", Extracting outliers_metrics, kpi_name="{kpi_name}", alert_lower_breached="{value_alert_lower_breached}", alert_upper_breached="{value_alert_upper_breached}"'
                             )
 
                         flx_outliers_metrics_parsed_msg = "Outliers Metrics JSON were submitted and successfully parsed"
@@ -1140,7 +1123,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
 
                 if flx_outliers_metrics and not flx_outliers_metrics_parsed:
                     logging.error(
-                        "failed to load the submitted outliers metrics as a JSON object, verify it is properly JSON encoded using single or double quotes delimitors"
+                        f'instance_id="{self.instance_id}", failed to load the submitted outliers metrics as a JSON object, verify it is properly JSON encoded using single or double quotes delimitors'
                     )
                     flx_outliers_metrics = flx_outliers_metrics
                     flx_outliers_metrics_parsed_msg = f'Outliers Metrics JSON were submitted but could not be parsed properly, verify the JSON syntax, properties should be enquoted with single or double quotes, exception="{flx_outliers_metrics_parsed_exception}"'
@@ -1172,7 +1155,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                         )
                     except Exception as e:
                         error_msg = f'Failed to convert the results to metrics with exception="{str(e)}"'
-                        logging.error(error_msg)
+                        logging.error(f'instance_id="{self.instance_id}", {error_msg}')
                         # do not raise an exception, continue
 
                 # process metrics_list, if any
@@ -1191,7 +1174,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                         error_msg = (
                             f'Failed to process metrics_list with exception="{str(e)}"'
                         )
-                        logging.error(error_msg)
+                        logging.error(f'instance_id="{self.instance_id}", {error_msg}')
                         # do not raise an exception, continue
 
                     # update the last seen metrics in the KVstore
@@ -1203,7 +1186,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                             )
                         except Exception as e:
                             error_msg = f'Failed to add last seen metrics record to batch list with exception="{str(e)}"'
-                            logging.error(error_msg)
+                            logging.error(f'instance_id="{self.instance_id}", {error_msg}')
 
             ########################
             # End Processing metrics
@@ -1387,6 +1370,20 @@ class TrackMeSplkFlxParse(StreamingCommand):
                 f'tenant_id="{self.tenant_id}", context="{self.context}", processed result="{json.dumps(flx_record, indent=2)}"'
             )
 
+        # end task
+        #
+        task_end = time.time()
+        task_run_time = round((task_end - task_start), 3)
+        logging.info(
+            f'instance_id={self.instance_id}, task="{task_name}", task_instance_id={task_instance_id}, task_run_time="{task_run_time}", task_end=1, task has terminated.'
+        )
+
+        # set task
+        #
+        task_start = time.time()
+        task_instance_id = get_uuid()
+        task_name = "process_thresholds_records"
+
         # process the thresholds records update limiting to one rest call per threshold rule
         endpoint = f"{self._metadata.searchinfo.splunkd_uri}/services/trackme/v2/splk_flx/write/flx_thresholds_add"
 
@@ -1402,7 +1399,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                         endpoint,
                         data=json.dumps(data),
                         verify=False,
-                        timeout=300,
+                        timeout=600,
                     )
                     response.raise_for_status()
                     logging.info(
@@ -1413,16 +1410,31 @@ class TrackMeSplkFlxParse(StreamingCommand):
                         f'tenant_id="{self.tenant_id}", failed to add the default threshold, exception="{str(e)}", data="{json.dumps(data, indent=2)}"'
                     )
 
+        # end task
+        #
+        task_end = time.time()
+        task_run_time = round((task_end - task_start), 3)
+        logging.info(
+            f'instance_id={self.instance_id}, task="{task_name}", task_instance_id={task_instance_id}, task_run_time="{task_run_time}", task_end=1, task has terminated.'
+        )
+
+        # set task
+        #
+        task_start = time.time()
+        task_instance_id = get_uuid()
+        task_name = "process_disruption_queue_records"
+
         # process the disruption queue records batch update
         if disruption_queue_records_to_add:
             try:
-                # Convert list to dictionary format for batch_kvstore_update
-                disruption_queue_dict = {
-                    record["_key"]: record for record in disruption_queue_records_to_add
-                }
-                self.batch_kvstore_update(
+                batch_update_worker(
                     f"kv_trackme_common_disruption_queue_tenant_{self.tenant_id}",
-                    disruption_queue_dict,
+                    self.service.kvstore[f"kv_trackme_common_disruption_queue_tenant_{self.tenant_id}"],
+                    disruption_queue_records_to_add,
+                    self.instance_id,
+                    get_uuid(),
+                    task_name="disruption_queue_update",
+                    max_multi_thread_workers=max_multi_thread_workers,
                 )
                 logging.info(
                     f'tenant_id="{self.tenant_id}", disruption queue records batch update completed successfully for {len(disruption_queue_records_to_add)} records'
@@ -1432,6 +1444,20 @@ class TrackMeSplkFlxParse(StreamingCommand):
                     f'tenant_id="{self.tenant_id}", failed to process the disruption queue records batch update, exception="{str(e)}"'
                 )
 
+        # end task
+        #
+        task_end = time.time()
+        task_run_time = round((task_end - task_start), 3)
+        logging.info(
+            f'instance_id={self.instance_id}, task="{task_name}", task_instance_id={task_instance_id}, task_run_time="{task_run_time}", task_end=1, task has terminated.'
+        )
+
+        # set task
+        #
+        task_start = time.time()
+        task_instance_id = get_uuid()
+        task_name = "process_drilldown_searches_records"
+
         # process the drilldown searches records if needed, run a single POST call to the endpoint
         if drilldown_searches_records_to_add:
 
@@ -1447,7 +1473,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                         "drilldown_records": drilldown_searches_records_to_add,
                     }),
                     verify=False,
-                    timeout=300,
+                    timeout=600,
                 )
                 response.raise_for_status()
                 logging.info(
@@ -1458,6 +1484,20 @@ class TrackMeSplkFlxParse(StreamingCommand):
                     f'tenant_id="{self.tenant_id}", failed to process the drilldown searches records batch update, exception="{str(e)}", data="{json.dumps(drilldown_searches_records_to_add, indent=2)}"'
                 )
 
+        # end task
+        #
+        task_end = time.time()
+        task_run_time = round((task_end - task_start), 3)
+        logging.info(
+            f'instance_id={self.instance_id}, task="{task_name}", task_instance_id={task_instance_id}, task_run_time="{task_run_time}", task_end=1, task has terminated.'
+        )
+
+        # set task
+        #
+        task_start = time.time()
+        task_instance_id = get_uuid()
+        task_name = "process_default_metrics_records"
+
         # process the default metrics records if needed, run a single POST call to the endpoint
         if default_metrics_records_to_add:
 
@@ -1473,7 +1513,7 @@ class TrackMeSplkFlxParse(StreamingCommand):
                         "default_metric_records": default_metrics_records_to_add,
                     }),
                     verify=False,
-                    timeout=300,
+                    timeout=600,
                 )
                 response.raise_for_status()
                 logging.info(
@@ -1484,17 +1524,36 @@ class TrackMeSplkFlxParse(StreamingCommand):
                     f'tenant_id="{self.tenant_id}", failed to process the default metrics records batch update, exception="{str(e)}", data="{json.dumps(default_metrics_records_to_add, indent=2)}"'
                 )
 
+        # end task
+        #
+        task_end = time.time()
+        task_run_time = round((task_end - task_start), 3)
+        logging.info(
+            f'instance_id={self.instance_id}, task="{task_name}", task_instance_id={task_instance_id}, task_run_time="{task_run_time}", task_end=1, task has terminated.'
+        )
+
+        # set task
+        #
+        task_start = time.time()
+        task_instance_id = get_uuid()
+        task_name = "process_last_seen_metrics_records"
+
         # process the last seen metrics records batch update
         if last_seen_metrics_records_to_add:
             try:
-                # Convert list of dictionaries to a single merged dictionary
+                # Convert nested structure to flat dictionary format for batch_update_worker
                 last_seen_metrics_dict = {}
-                for record_dict in last_seen_metrics_records_to_add:
-                    last_seen_metrics_dict.update(record_dict)
-
-                self.batch_kvstore_update(
+                for record in last_seen_metrics_records_to_add:
+                    last_seen_metrics_dict.update(record)
+                
+                batch_update_worker(
                     f"kv_trackme_flx_last_seen_activity_tenant_{self.tenant_id}",
+                    self.service.kvstore[f"kv_trackme_flx_last_seen_activity_tenant_{self.tenant_id}"],
                     last_seen_metrics_dict,
+                    self.instance_id,
+                    get_uuid(),
+                    task_name="last_seen_activity_update",
+                    max_multi_thread_workers=max_multi_thread_workers,
                 )
                 logging.info(
                     f'tenant_id="{self.tenant_id}", last seen metrics records batch update completed successfully for {len(last_seen_metrics_records_to_add)} records'
@@ -1504,10 +1563,18 @@ class TrackMeSplkFlxParse(StreamingCommand):
                     f'tenant_id="{self.tenant_id}", failed to process the last seen metrics records batch update, exception="{str(e)}"'
                 )
 
+        # end task
+        #
+        task_end = time.time()
+        task_run_time = round((task_end - task_start), 3)
+        logging.info(
+            f'instance_id={self.instance_id}, task="{task_name}", task_instance_id={task_instance_id}, task_run_time="{task_run_time}", task_end=1, task has terminated.'
+        )
+
         # log info
         run_time = round(time.time() - start, 3)
         logging.info(
-            f'tenant_id="{self.tenant_id}", context="{self.context}", TrackMeSplkFlxParse has terminated successfully, turn debug mode on for more details, results_count="{count}", run_time={run_time}'
+            f'tenant_id="{self.tenant_id}", context="{self.context}", instance_id="{self.instance_id}", TrackMeSplkFlxParse has terminated successfully, turn debug mode on for more details, results_count="{count}", run_time={run_time}'
         )
 
 
